@@ -466,18 +466,17 @@ class AzureBlobFileSystem(AsyncFileSystem):
         """
         try:
             self.account_url: str = f"https://{self.account_name}.blob.core.windows.net"
-            if self.credential is not None:
-                self.service_client = AIOBlobServiceClient(
-                    account_url=self.account_url, credential=self.credential
-                )
+            creds = [self.credential, self.account_key]
+            if any(creds):
+                self.service_client = [
+                    AIOBlobServiceClient(account_url=self.account_url, credential=cred)
+                    for cred in creds
+                    if cred is not None
+                ][0]
             elif self.connection_string is not None:
                 self.service_client = AIOBlobServiceClient.from_connection_string(
                     conn_str=self.connection_string
                 )
-            elif self.account_key is not None:
-                self.service_client = AIOBlobServiceClient(
-                    account_url=self.account_url, credential=self.account_key
-                )
             elif self.sas_token is not None:
                 self.service_client = AIOBlobServiceClient(
                     account_url=self.account_url + self.sas_token, credential=None
@@ -602,7 +601,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
             root = ""
             depth = 20 if "**" in path else 1
 
-        allpaths = await self._find(
+        allpaths = await self._glob_find(
             root, maxdepth=depth, withdirs=True, detail=True, **kwargs
         )
         pattern = (
@@ -701,7 +700,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 containers = [c async for c in contents]
                 files = await self._details(containers)
                 self.dircache[path] = files
-                return files
+
             return self.dircache[path]
         else:
             if target_path not in self.dircache or invalidate_cache or return_glob:
@@ -746,22 +745,18 @@ class AzureBlobFileSystem(AsyncFileSystem):
                                             pass
                     except ResourceNotFoundError:
                         raise FileNotFoundError
+                    finalblobs = await self._details(
+                        outblobs, target_path=target_path, return_glob=return_glob
+                    )
                     if return_glob:
-                        finalblobs = await self._details(
-                            outblobs, target_path=target_path, return_glob=True
-                        )
                         return finalblobs
-                    else:
-                        finalblobs = await self._details(
-                            outblobs, target_path=target_path
-                        )
+                    finalblobs = await self._details(outblobs, target_path=target_path)
                     if not finalblobs:
                         if not await self._exists(target_path):
                             raise FileNotFoundError
-                        else:
-                            return []
+                        return []
                     self.dircache[target_path] = finalblobs
-                    return finalblobs
+
             return self.dircache[target_path]
 
     async def _details(
@@ -823,16 +818,65 @@ class AzureBlobFileSystem(AsyncFileSystem):
             output = await filter_blobs(output, target_path)
         return output
 
-    def find(self, path, maxdepth=None, withdirs=False, **kwargs):
+    def find(self, path, withdirs=False, prefix="", **kwargs):
         return maybe_sync(
-            self._find, self, path=path, maxdepth=maxdepth, withdirs=withdirs, **kwargs
+            self._find, self, path=path, withdirs=withdirs, prefix=prefix, **kwargs
         )
 
-    async def _find(self, path, maxdepth=None, withdirs=False, **kwargs):
+    async def _find(self, path, withdirs=False, prefix="", **kwargs):
         """List all files below path.
         Like posix ``find`` command without conditions
         Parameters
         ----------
+        path : str
+            The path (directory) to list from
+        withdirs: bool
+            Whether to include directory paths in the output. This is True
+            when used by glob, but users usually only want files.
+        prefix: str
+            Only return files that match `^{path}/{prefix}`
+        kwargs are passed to ``ls``.
+        """
+        path = self._strip_protocol(path)
+        parent_path = path.strip("/") + "/"
+        target_path = f"{parent_path}{(prefix or '').lstrip('/')}"
+        # target_path = path.strip("/")
+        container, path = self.split_path(target_path)
+
+        container_client = self.service_client.get_container_client(container=container)
+        blobs = container_client.list_blobs(include=["metadata"], name_starts_with=path)
+        files = {}
+        dir_set = set()
+        dirs = {}
+        detail = kwargs.pop("detail", False)
+        try:
+            infos = await self._details([b async for b in blobs])
+            for info in infos:
+                name = info["name"]
+                parent_dir = self._parent(name)
+                if parent_dir not in dir_set and parent_dir != parent_path.strip("/"):
+                    dir_set.add(parent_dir)
+                    dirs[parent_dir] = {
+                        "name": parent_dir,
+                        "type": "directory",
+                        "size": 0,
+                    }
+                files[name] = info
+        except ResourceNotFoundError:
+            # find doesn't raise but returns [] or {} instead
+            pass
+        if withdirs:
+            files.update(dirs)
+        names = sorted(files)
+        if not detail:
+            return names
+        return {name: files[name] for name in names}
+
+    async def _glob_find(self, path, maxdepth=None, withdirs=False, **kwargs):
+        """List all files below path in a recusrsive manner.
+        Like posix ``find`` command without conditions
+        Parameters
+        ----------
         path : str
         maxdepth: int or None
             If not None, the maximum number of levels to descend
@@ -1139,12 +1183,19 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 return True
         except FileNotFoundError:
             pass
-        try:
-            await self._info(path)
+
+        container_name, path = self.split_path(path)
+
+        if not path:
+            # Empty paths exist by definition
             return True
-        except:  # noqa: E722
-            # any exception allowed bar FileNotFoundError?
-            return False
+
+        cc = self.service_client.get_container_client(container_name)
+        bc = cc.get_blob_client(blob=path)
+        async with bc:
+            exists = await bc.exists()
+
+        return exists
 
     def cat(self, path, recursive=False, on_error="raise", **kwargs):
         """Fetch (potentially multiple) paths' contents
@@ -1464,18 +1515,19 @@ class AzureBlobFile(io.IOBase):
             self.fs.account_url: str = (
                 f"https://{self.fs.account_name}.blob.core.windows.net"
             )
-            if self.fs.sync_credential is not None:
-                self.container_client = BlobServiceClient(
-                    account_url=self.fs.account_url, credential=self.fs.sync_credential
-                ).get_container_client(self.container_name)
+            creds = [self.fs.sync_credential, self.fs.account_key, self.fs.credential]
+            if any(creds):
+                self.container_client = [
+                    BlobServiceClient(
+                        account_url=self.fs.account_url, credential=cred
+                    ).get_container_client(self.container_name)
+                    for cred in creds
+                    if cred is not None
+                ][0]
             elif self.fs.connection_string is not None:
                 self.container_client = BlobServiceClient.from_connection_string(
                     conn_str=self.fs.connection_string
                 ).get_container_client(self.container_name)
-            elif self.fs.account_key is not None:
-                self.container_client = BlobServiceClient(
-                    account_url=self.fs.account_url, credential=self.fs.account_key
-                ).get_container_client(self.container_name)
             elif self.fs.sas_token is not None:
                 self.container_client = BlobServiceClient(
                     account_url=self.fs.account_url + self.fs.sas_token, credential=None
@@ -1486,7 +1538,9 @@ class AzureBlobFile(io.IOBase):
                 ).get_container_client(self.container_name)
 
         except Exception as e:
-            raise ValueError(f"Unable to connect with provided params for {e}!!")
+            raise ValueError(
+                f"Unable to fetch container_client with provided params for {e}!!"
+            )
 
     @property
     def closed(self):
@@ -1613,10 +1667,17 @@ class AzureBlobFile(io.IOBase):
                 # This step handles the situation where data="" and length=0
                 # which is throws an InvalidHeader error from Azure, so instead
                 # of staging a block, we directly upload the empty blob
+                # This isn't actually tested, since Azureite behaves differently.
                 if block_id == "0000000" and length == 0 and final:
                     self.blob_client.upload_blob(data=data, metadata=self.metadata)
+                elif length == 0 and final:
+                    # just finalize
+                    block_list = [BlobBlock(_id) for _id in self._block_list]
+                    self.blob_client.commit_block_list(
+                        block_list=block_list, metadata=self.metadata
+                    )
                 else:
-                    raise RuntimeError(f"Failed to upload block with {e}!!")
+                    raise RuntimeError(f"Failed to upload block{e}!") from e
         elif self.mode == "ab":
             self.blob_client.upload_blob(
                 data=data,
